@@ -3,13 +3,13 @@ from symengine import (Symbol, Integer, sympify, SympifyError, log,
         exp, gamma, have_mpfr, have_mpc, DenseMatrix, sin, cos, tan, cot,
         csc, sec, asin, acos, atan, acot, acsc, asec, sinh, cosh, tanh, coth,
         asinh, acosh, atanh, acoth, Add, Mul, Pow, diff, GoldenRatio,
-        Catalan, EulerGamma)
+        Catalan, EulerGamma, UnevaluatedExpr)
 from symengine.lib.symengine_wrapper import (Subs, Derivative, RealMPFR,
         ComplexMPC, PyNumber, Function, LambertW, zeta, dirichlet_eta,
         KroneckerDelta, LeviCivita, erf, erfc, lowergamma, uppergamma,
         loggamma, beta, polygamma, sign, floor, ceiling, conjugate, And,
         Or, Not, Xor, Piecewise, Interval, EmptySet, FiniteSet, Contains,
-        Union, Complement, UniversalSet)
+        Union, Complement, UniversalSet, Reals, Rationals, Integers)
 import unittest
 
 # Note: We test _sympy_() for SymEngine -> SymPy conversion, as those are
@@ -651,6 +651,15 @@ def test_conjugate():
     assert e2._sympy_() == e1
 
 
+@unittest.skipIf(not have_sympy, "SymPy not installed")
+def test_unevaluated_expr():
+    x = Symbol("x")
+    e1 = sympy.UnevaluatedExpr(sympy.Symbol("x"))
+    e2 = UnevaluatedExpr(x)
+    assert sympify(e1) == e2
+    assert e2._sympy_() == e1
+
+
 @unittest.skipIf(not have_sympy, "SymPy not installed")
 def test_logic():
     x = true
@@ -709,6 +718,15 @@ def test_sets():
     assert sympify(sympy.S.UniversalSet) == UniversalSet()
     assert sympy.S.UniversalSet == UniversalSet()._sympy_()
 
+    assert sympify(sympy.S.Reals) == Reals()
+    assert sympy.S.Reals == Reals()._sympy_()
+
+    assert sympify(sympy.S.Rationals) == Rationals()
+    assert sympy.S.Rationals == Rationals()._sympy_()
+
+    assert sympify(sympy.S.Integers) == Integers()
+    assert sympy.S.Integers == Integers()._sympy_()
+
     assert FiniteSet(x, y) == FiniteSet(x1, y1)
     assert FiniteSet(x1, y) == FiniteSet(x1, y1)
     assert FiniteSet(x, y)._sympy_() == sympy.FiniteSet(x1, y1)
@@ -768,3 +786,12 @@ def test_pynumber():
 
     b = b / x
     assert isinstance(b, PyNumber)
+
+
+@unittest.skipIf(not have_sympy, "SymPy not installed")
+def test_construct_dense_matrix():
+    # Test for issue #347
+    A = sympy.Matrix([[1, 2], [3, 5]])
+    B = DenseMatrix(A)
+    assert B.shape == (2, 2)
+    assert list(B) == [1, 2, 3, 5]
